unit mimetype;

interface

uses
  system.Classes;

type
  TMimeExtPair = class (TObject)
  private
    FKey, FValue : String;
  protected
  public
    constructor create(_key, _value : string);
  published
    property key   : String read FKey;
    property value : String read FValue;
  end;

  TMimeMap = class (TObject)
  private
    FMimeList : TStringList;
    FExtList  : TStringList;
    procedure putMime(_mime, _ext : String);
    procedure putExt(_ext, _mime : String);
    procedure setUpMimes;
    procedure setUpExts;
  protected
  public
    constructor create;
    destructor destroy;
    function getMimeType(_ext : String) : string;
    function getMimeTypeFromFileName(_fileName : String) : string;
    function getExtension(_mime : String) : string;
    function getExtensionFromFileName(_fileName : String) : string;
  end;

implementation

uses
  System.StrUtils, System.Types;

// TMimeExtPair = class (TObject)
//  private
//  protected
//  public
constructor TMimeExtPair.create(_key, _value : string);
begin
  inherited create;
  FKey   := _key;;
  FValue := _value
end;

//  TMimeMap = class (TObject)
//  private
procedure TMimeMap.putMime(_mime, _ext : String);
begin
  FMimeList.AddObject(_mime, TMimeExtPair.create(_mime, _ext));
end;

procedure TMimeMap.putExt(_ext: string; _mime: string);
begin
  FExtList.AddObject(_ext, TMimeExtPair.create(_ext, _mime));
end;

procedure TMimeMap.setUpMimes;
begin
  putMime('application/acad','dwg');
  putMime('application/astound','asn');
  putMime('application/dsptype','tsp');
  putMime('application/dxf','dxf');
  putMime('application/force-download','reg');
  putMime('application/futuresplash','spl');
  putMime('application/gzip','gz');
  putMime('application/javascript','js');
  putMime('application/json','json');
  putMime('application/listenup','ptlk');
  putMime('application/mac-binhex40','hqx');
  putMime('application/mbedlet','mbd');
  putMime('application/mif','mif');
  putMime('application/msexcel','xls ');
  putMime('application/mshelp','hlp');
  putMime('application/mspowerpoint','ppt');
  putMime('application/msword','doc');
  putMime('application/octet-stream','bin');
  putMime('application/oda','oda');
  putMime('application/pdf','pdf');
  putMime('application/postscript','ps');
  putMime('application/rtc','rtc');
  putMime('application/rtf','rtf');
  putMime('application/studiom','smp');
  putMime('application/toolbook','tbk');
  putMime('application/vnd.oasis.opendocument.chart','odc');
  putMime('application/vnd.oasis.opendocument.formula','odf');
  putMime('application/vnd.oasis.opendocument.graphics','odg');
  putMime('application/vnd.oasis.opendocument.image','odi');
  putMime('application/vnd.oasis.opendocument.presentation','odp');
  putMime('application/vnd.oasis.opendocument.spreadsheet','ods');
  putMime('application/vnd.oasis.opendocument.text','odt');
  putMime('application/vnd.oasis.opendocument.text-master','odm');
  putMime('application/vocaltec-media-desc','vmd');
  putMime('application/vocaltec-media-file','vmf');
  putMime('application/vnd.openxmlformats-officedocument.presentationml.presentation','pptx');
  putMime('application/vnd.openxmlformats-officedocument.spreadsheetml.sheet','xlsx');
  putMime('application/vnd.openxmlformats-officedocument.wordprocessingml.document','docx');
  putMime('application/xhtml+xml','html');
  putMime('application/xml','xml');
  putMime('application/x-bcpio','bcpio');
  putMime('application/x-compress','z');
  putMime('application/x-cpio','cpio');
  putMime('application/x-csh','csh');
  putMime('application/x-director','dcr');
  putMime('application/x-dvi','dvi');
  putMime('application/x-envoy','evy');
  putMime('application/x-gtar','gtar');
  putMime('application/x-hdf','hdf');
  putMime('application/x-httpd-php','php');
  putMime('application/x-latex','latex');
  putMime('application/x-macbinary','bin');
  putMime('application/x-mif','mif');
  putMime('application/x-netcdf','cdf');
  putMime('application/x-sh','sh');
  putMime('application/x-shar','shar');
  putMime('application/x-shockwave-flash','swf');
  putMime('application/x-sprite','spr');
  putMime('application/x-stuffit','sit');
  putMime('application/x-supercard','sca');
  putMime('application/x-sv4cpio','sv4cpio');
  putMime('application/x-sv4crc','sv4crc');
  putMime('application/x-tar','tar');
  putMime('application/x-tcl','tcl');
  putMime('application/x-tex','tex');
  putMime('application/x-texinfo','texinfo');
  putMime('application/x-troff','troff');
  putMime('application/x-troff-man','man');
  putMime('application/x-troff-me','me');
  putMime('application/x-ustar','ustar');
  putMime('application/x-wais-source','src');
  putMime('application/zip','zip');
  putMime('audio/basic','snd');
  putMime('audio/echospeech','es');
  putMime('audio/mpeg','mp3');
  putMime('audio/mp4','mp4');
  putMime('audio/ogg','ogg');
  putMime('audio/opus','opus');
  putMime('audio/tsplayer','tsi');
  putMime('audio/voxware','vox');
  putMime('audio/wav','wav');
  putMime('audio/x-aiff','aiff');
  putMime('audio/x-dspeech','cht');
  putMime('audio/x-midi','midi');
  putMime('audio/x-mpeg','mp2');
  putMime('audio/x-pn-realaudio','ram');
  putMime('audio/x-pn-realaudio-plugin','rpm');
  putMime('audio/x-qt-stream','stream');
  putMime('drawing/x-dwf','dwf');
  putMime('image/bmp','bmp');
  putMime('image/x-bmp','bmp');
  putMime('image/x-ms-bmp','bmp');
  putMime('image/cis-cod','cod');
  putMime('image/cmu-raster','ras');
  putMime('image/fif','fif');
  putMime('image/gif','gif');
  putMime('image/ief','ief');
  putMime('image/jpeg','jpg');
  putMime('image/png','png');
  putMime('image/svg+xml','svg');
  putMime('image/tiff','tiff');
  putMime('image/vasa','mcf');
  putMime('image/vnd.wap.wbmp','wbmp');
  putMime('image/x-freehand','fhc');
  putMime('image/x-icon','ico');
  putMime('image/x-portable-anymap','pnm');
  putMime('image/x-portable-bitmap','pbm');
  putMime('image/x-portable-graymap','pgm');
  putMime('image/x-portable-pixmap','ppm');
  putMime('image/x-rgb','rgb');
  putMime('image/x-windowdump','xwd');
  putMime('image/x-xbitmap','xbm');
  putMime('image/x-xpixmap','xpm');
  putMime('text/calendar','ics');
  putMime('text/csv','csv');
  putMime('text/css','css');
  putMime('text/html','html');
  putMime('text/javascript','js');
  putMime('text/markdown','md');
  putMime('text/plain','txt');
  putMime('text/richtext','rtx');
  putMime('text/rtf','rtf');
  putMime('text/tab-separated-values','tsv');
  putMime('text/vnd.wap.wml','wml');
  putMime('application/vnd.wap.wmlc','wmlc');
  putMime('text/vnd.wap.wmlscript','wmls');
  putMime('application/vnd.wap.wmlscriptc','wmlsc');
  putMime('text/xml','xml');
  putMime('text/x-setext','etx');
  putMime('text/x-sgml','sgml');
  putMime('text/x-speech','talk');
  putMime('video/mpeg','mpeg');
  putMime('video/mpeg','mpg');
  putMime('video/mpeg','mpe');
  putMime('video/mp4','mp4');
  putMime('video/ogg','ogg');
  putMime('video/quicktime','mov');
  putMime('video/vnd.vivo','viv');
  putMime('video/webm','webm');
  putMime('video/x-msvideo','avi');
  putMime('video/x-sgi-movie','movie');
  putMime('video/3gpp','3gp');
  putMime('workbook/formulaone','vts');
  putMime('x-world/x-3dmf','qd3');
  putMime('x-world/x-vrml','wrl');
end;

procedure TMimeMap.setUpExts;
begin
  putExt('mf','x-world/x-3dmf');
  putExt('m','x-world/x-3dmf');
  putExt('zip','application/zip');
  putExt('z','application/x-compress');
  putExt('xwd','image/x-windowdump');
  putExt('xpm','image/x-xpixmap');
  putExt('xml','text/xml');
  putExt('xlsx','application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
  putExt('xls ','application/msexcel');
  putExt('xla','application/msexcel');
  putExt('xhtml','application/xhtml+xml');
  putExt('xbm','image/x-xbitmap');
  putExt('wrl','x-world/x-vrml');
  putExt('wmlsc','application/vnd.wap.wmlscriptc');
  putExt('wmls','text/vnd.wap.wmlscript');
  putExt('wmlc','application/vnd.wap.wmlc');
  putExt('wml','text/vnd.wap.wml');
  putExt('webm','video/webm');
  putExt('wbmp','image/vnd.wap.wbmp');
  putExt('wav','audio/wav');
  putExt('vtts','workbook/formulaone');
  putExt('vts','workbook/formulaone');
  putExt('vox','audio/voxware');
  putExt('vmf','application/vocaltec-media-file');
  putExt('vmd','application/vocaltec-media-desc');
  putExt('vivo','video/vnd.vivo');
  putExt('viv','video/vnd.vivo');
  putExt('ustar','application/x-ustar');
  putExt('txt','text/plain');
  putExt('tsv','text/tab-separated-values');
  putExt('tsp','application/dsptype');
  putExt('tsi','audio/tsplayer');
  putExt('troff','application/x-troff');
  putExt('tr','application/x-troff');
  putExt('tiff','image/tiff');
  putExt('tif','image/tiff');
  putExt('texinfo','application/x-texinfo');
  putExt('texi','application/x-texinfo');
  putExt('tex','application/x-tex');
  putExt('tcl','application/x-tcl');
  putExt('tbk','application/toolbook');
  putExt('tar','application/x-tar');
  putExt('talk','text/x-speech');
  putExt('t','application/x-troff');
  putExt('swf','application/x-shockwave-flash');
  putExt('svg','image/svg+xml');
  putExt('sv4crc','application/x-sv4crc');
  putExt('sv4cpio','application/x-sv4cpio');
  putExt('stream','audio/x-qt-stream');
  putExt('src','application/x-wais-source');
  putExt('sprite','application/x-sprite');
  putExt('spr','application/x-sprite');
  putExt('spl','application/futuresplash');
  putExt('spc','text/x-speech');
  putExt('snd','audio/basic');
  putExt('smp','application/studiom');
  putExt('sit','application/x-stuffit');
  putExt('shtml','text/html');
  putExt('shar','application/x-shar');
  putExt('sh','application/x-sh');
  putExt('sgml','text/x-sgml');
  putExt('sgm','text/x-sgml');
  putExt('sca','application/x-supercard');
  putExt('rtx','text/richtext');
  putExt('rtf','text/rtf');
  putExt('rtc','application/rtc');
  putExt('rpm','audio/x-pn-realaudio-plugin');
  putExt('rgb','image/x-rgb');
  putExt('reg','application/force-download');
  putExt('ras','image/cmu-raster');
  putExt('ram','audio/x-pn-realaudio');
  putExt('ra','audio/x-pn-realaudio');
  putExt('qt','video/quicktime');
  putExt('qd3d','x-world/x-3dmf');
  putExt('qd3','x-world/x-3dmf');
  putExt('ptlk','application/listenup');
  putExt('ps','application/postscript');
  putExt('ppz','application/mspowerpoint');
  putExt('pptx','application/vnd.openxmlformats-officedocument.presentationml.presentation');
  putExt('ppt','application/mspowerpoint');
  putExt('pps','application/mspowerpoint');
  putExt('ppm','image/x-portable-pixmap');
  putExt('pot','application/mspowerpoint');
  putExt('pnm','image/x-portable-anymap');
  putExt('png','image/png');
  putExt('phtml','application/x-httpd-php');
  putExt('php','application/x-httpd-php');
  putExt('pgm','image/x-portable-graymap');
  putExt('pdf','application/pdf');
  putExt('pbm','image/x-portable-bitmap');
  putExt('opus','audio/opus');
  putExt('ogv','video/ogg');
  putExt('ogg','video/ogg');
  putExt('odt','application/vnd.oasis.opendocument.text');
  putExt('ods','application/vnd.oasis.opendocument.spreadsheet');
  putExt('odp','application/vnd.oasis.opendocument.presentation');
  putExt('odm','application/vnd.oasis.opendocument.text-master');
  putExt('odi','application/vnd.oasis.opendocument.image');
  putExt('odg','application/vnd.oasis.opendocument.graphics');
  putExt('odf','application/vnd.oasis.opendocument.formula');
  putExt('odc','application/vnd.oasis.opendocument.chart');
  putExt('oda','application/oda');
  putExt('nsc','application/x-nschat');
  putExt('nc','application/x-netcdf');
  putExt('mpg','video/mpeg');
  putExt('mpeg','video/mpeg');
  putExt('mpe','video/mpeg');
  putExt('mp4','video/mp4');
  putExt('mp3','audio/mpeg');
  putExt('mp2','audio/x-mpeg');
  putExt('movie','video/x-sgi-movie');
  putExt('mov','video/quicktime');
  putExt('mif','application/mif');
  putExt('midi','audio/x-midi');
  putExt('mid','audio/x-midi');
  putExt('me','application/x-troff-me');
  putExt('md','text/markdown');
  putExt('mcf','image/vasa');
  putExt('mbd','application/mbedlet');
  putExt('markdown','text/markdown');
  putExt('man','application/x-troff-man');
  putExt('latex','application/x-latex');
  putExt('json','application/json');
  putExt('js','text/javascript');
  putExt('jpg','image/jpeg');
  putExt('jpeg','image/jpeg');
  putExt('jpe','image/jpeg');
  putExt('ini','application/octet-stream');
  putExt('ief','image/ief');
  putExt('ics','text/calendar');
  putExt('ico','image/x-icon');
  putExt('html','text/html');
  putExt('htm','text/html');
  putExt('hqx','application/mac-binhex40');
  putExt('hlp','application/mshelp');
  putExt('hdf','application/x-hdf');
  putExt('gz','application/gzip');
  putExt('gtar','application/x-gtar');
  putExt('gif','image/gif');
  putExt('file','application/octet-stream');
  putExt('fif','image/fif');
  putExt('fhc','image/x-freehand');
  putExt('fh5','image/x-freehand');
  putExt('fh4','image/x-freehand');
  putExt('evy','application/x-envoy');
  putExt('etx','text/x-setext');
  putExt('es','audio/echospeech');
  putExt('eps','application/postscript');
  putExt('dxr','application/x-director');
  putExt('dxf','application/dxf');
  putExt('dwg','application/acad');
  putExt('dwf','drawing/x-dwf');
  putExt('dvi','application/x-dvi');
  putExt('dus','audio/x-dspeech');
  putExt('dot','application/msword');
  putExt('doc','application/msword');
  putExt('docx','application/vnd.openxmlformats-officedocument.wordprocessingml.document');
  putExt('dir','application/x-director');
  putExt('dcr','application/x-director');
  putExt('csv','text/csv');
  putExt('css','text/css');
  putExt('csh','application/x-csh');
  putExt('cpio','application/x-cpio');
  putExt('com','application/octet-stream');
  putExt('cod','image/cis-cod');
  putExt('class','application/octet-stream');
  putExt('cht','audio/x-dspeech');
  putExt('chm','application/mshelp');
  putExt('cdf','application/x-netcdf');
  putExt('bmp','image/bmp');
  putExt('bin','application/x-macbinary');
  putExt('bcpio','application/x-bcpio');
  putExt('avi','video/x-msvideo');
  putExt('au','audio/basic');
  putExt('asn','application/astound');
  putExt('asd','application/astound');
  putExt('aiff','audio/x-aiff');
  putExt('aifc','audio/x-aiff');
  putExt('aif','audio/x-aiff');
  putExt('ai','application/postscript');
  putExt('3gp','video/3gpp');
end;

//  protected
//  public
constructor TMimeMap.create;
begin
  inherited create;
  FMimeList := TStringList.Create;
  FExtList  := TStringList.Create;
  setUpMimes;
  setUpExts;
end;

destructor TMimeMap.destroy;
var
  i : integer;
begin
  for i := 0 to FMimeList.Count -1 do
    FMimeList.Objects[i].Free;
  for i := 0 to FExtList.Count -1 do
    FExtList.Objects[i].Free;
  FMimeList.free;
  FExtList.free;
end;

function TMimeMap.getMimeType(_ext : String) : string;
begin
  result := 'application/octet-stream';
  if FExtList.IndexOf(_ext) >= 0 then
    result := (FExtList.objects[FExtList.IndexOf(_ext)] as TMimeExtPair).value;
end;

function TMimeMap.getExtension(_mime : String) : string;
begin
  result := '';
  if FMimeList.IndexOf(_mime) >= 0 then
  result := (FMimeList.objects[FMimeList.IndexOf(_mime)] as TMimeExtPair).value;
end;

function TMimeMap.getMimeTypeFromFileName(_fileName : String) : string;
begin
  if Pos('/document/image', _fileName) > 0  then
    result := 'image/JPEG'
  else
    result := getMimeType(getExtensionFromFileName(_fileName));
end;

function TMimeMap.getExtensionFromFileName(_fileName : String) : string;
var
  parts : TStringDynArray;
begin
  parts := SplitString(_fileName, '.');
  result := parts[length(parts)-1];
end;

end.
