unit frmMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Memo.Types,
  FMX.StdCtrls, FMX.Controls.Presentation, FMX.ScrollBox, FMX.Memo, filepicker,
  FMX.Layouts, FMX.ExtCtrls, FMX.Menus;

type
  TLoadFileThread=class(TThread)
  private
    FFilePicker    : TFilePicker;
    FImage         : TBitmap;
    FFileContent64 : string;
  protected
    procedure Execute;override;
  public
    constructor Create(_filePicker : TFilePicker);
    destructor Destroy; override;
    property fileContent : string  read FFileContent64;
    property image       : TBitmap read FImage;
  end;

  TfMain = class(TForm)
    StyleBook1: TStyleBook;
    memContent: TMemo;
    btnPick: TButton;
    lblFileName: TLabel;
    dlgOpen: TOpenDialog;
    lblMimeType: TLabel;
    ImageViewer1: TImageViewer;
    AniIndicator1: TAniIndicator;
    procedure FormCreate(Sender: TObject);
    procedure btnPickClick(Sender: TObject);
  private
    FLoadFileThread : TLoadFileThread;
    FFilePicker : TFilePicker;
    procedure listenFilePicker(const _picked : boolean);
    property FilePicker : TFilePicker read FFilePicker;
    procedure OnTerminate(Sender:TObject);
  public
    { Public-Deklarationen }
  end;

var
  fMain: TfMain;

implementation
uses
  System.NetEncoding, System.threading;

{$R *.fmx}

//  TLoadFileThread=class(TThread)
//  private
//  protected
procedure TLoadFileThread.Execute;
var
  ms : TMemoryStream;
  imageBytes : TBytes;
  mime : String;
begin
  if FFilePicker.hasPickedFile then begin
    imageBytes := FFilePicker.getPickedFileBytes;
    FFileContent64 := TNetEncoding.Base64.EncodeBytesToString(imageBytes);
    mime := FFilePicker.pickedFileMimeType;
    if mime.StartsText('image', mime) then begin
      ms := TMemoryStream.Create;
      try
        ms.Write(imageBytes[0], length(imageBytes));
        FImage := TBitmap.Create;
        FImage.LoadFromStream(ms);
      finally
        ms.free;
      end;
    end;
  end;
end;

//  public
constructor TLoadFileThread.Create(_filePicker : TFilePicker);
begin
  FFilePicker := _filePicker;
  inherited Create(True);
end;

destructor TLoadFileThread.destroy;
begin
  if FImage <> nil then
    FreeAndNil(FImage);
  inherited;
end;

procedure TfMain.listenFilePicker(const _picked : boolean);
begin
  if _picked then begin
    AniIndicator1.Enabled := true;
    AniIndicator1.Visible := true;
    lblFileName.Text := FilePicker.pickedFileName;
    lblMimeType.Text := FilePicker.pickedFileExtension +' -- '+ FilePicker.pickedFileMimeType;
    Application.ProcessMessages;
    FLoadFileThread:= TLoadFileThread.Create(FFilePicker);
    FLoadFileThread.OnTerminate:=OnTerminate;
    FLoadFileThread.Resume;
 end;
end;

procedure TfMain.btnPickClick(Sender: TObject);
begin
  FilePicker.openFileFicker('*');
end;

procedure TfMain.FormCreate(Sender: TObject);
begin
  FFilePicker := TFilePicker.create(dlgOpen, '', listenFilePicker);
end;

procedure TfMain.OnTerminate(Sender:TObject);
begin
  memContent.Text := FLoadFileThread.fileContent;
  if FLoadFileThread.image <> nil then begin
    ImageViewer1.Bitmap.Assign(FLoadFileThread.image);
    ImageViewer1.BestFit;
    ImageViewer1.RealignContent;
  end;
  FLoadFileThread := nil;
  AniIndicator1.Enabled := false;
  AniIndicator1.Visible := false;
end;

end.
